@@ -23,6 +23,7 @@
 #ifdef SDL_JOYSTICK_HIDAPI
 
 #include "../../SDL_hints_c.h"
+#include "../../misc/SDL_libusb.h"
 #include "../SDL_sysjoystick.h"
 #include "SDL_hidapijoystick_c.h"
 #include "SDL_hidapi_rumble.h"
@@ -31,7 +32,9 @@
 #ifdef SDL_JOYSTICK_HIDAPI_GAMECUBE
 
 // Define this if you want to log all packets from the controller
-// #define DEBUG_GAMECUBE_PROTOCOL
+#if 0
+#define DEBUG_GAMECUBE_PROTOCOL
+#endif
 
 #define MAX_CONTROLLERS 4
 
@@ -74,7 +77,8 @@ static bool HIDAPI_DriverGameCube_IsSupportedDevice(SDL_HIDAPI_Device *device, c
     }
     if (vendor_id == USB_VENDOR_DRAGONRISE &&
         (product_id == USB_PRODUCT_EVORETRO_GAMECUBE_ADAPTER1 ||
-         product_id == USB_PRODUCT_EVORETRO_GAMECUBE_ADAPTER2)) {
+         product_id == USB_PRODUCT_EVORETRO_GAMECUBE_ADAPTER2 ||
+         product_id == USB_PRODUCT_EVORETRO_GAMECUBE_ADAPTER3)) {
         // EVORETRO GameCube Controller Adapter
         return true;
     }
@@ -99,6 +103,83 @@ static void SDLCALL SDL_JoystickGameCubeRumbleBrakeHintChanged(void *userdata, c
     }
 }
 
+static bool HIDAPI_DriverGameCube_EnableAdapter(SDL_HIDAPI_Device *device)
+{
+#ifdef HAVE_LIBUSB
+    // Need to close the device while sending USB commands to it
+    SDL_hid_close(device->dev);
+
+    // This is needed to enable input for Nyko and EVORETRO GameCube adapters
+    SDL_LibUSBContext *libusb_ctx;
+    if (SDL_InitLibUSB(&libusb_ctx)) {
+        libusb_context *context = NULL;
+        libusb_device **devs = NULL;
+        libusb_device_handle *handle = NULL;
+        struct libusb_device_descriptor desc;
+        ssize_t i, num_devs;
+        bool kernel_detached = false;
+
+        if (libusb_ctx->init(&context) == 0) {
+            num_devs = libusb_ctx->get_device_list(context, &devs);
+            for (i = 0; i < num_devs; ++i) {
+                if (libusb_ctx->get_device_descriptor(devs[i], &desc) != 0) {
+                    continue;
+                }
+
+                if (desc.idVendor != USB_VENDOR_NINTENDO ||
+                    desc.idProduct != USB_PRODUCT_NINTENDO_GAMECUBE_ADAPTER) {
+                    continue;
+                }
+
+                if (libusb_ctx->open(devs[i], &handle) != 0) {
+                    continue;
+                }
+
+                if (libusb_ctx->kernel_driver_active(handle, 0)) {
+                    if (libusb_ctx->detach_kernel_driver(handle, 0) == 0) {
+                        kernel_detached = true;
+                    }
+                }
+
+                if (libusb_ctx->claim_interface(handle, 0) == 0) {
+                    libusb_ctx->control_transfer(handle, 0x21, 11, 0x0001, 0, NULL, 0, 1000);
+                    libusb_ctx->release_interface(handle, 0);
+                }
+
+                if (kernel_detached) {
+                    libusb_ctx->attach_kernel_driver(handle, 0);
+                }
+
+                libusb_ctx->close(handle);
+            }
+
+            libusb_ctx->free_device_list(devs, 1);
+
+            libusb_ctx->exit(context);
+        }
+        SDL_QuitLibUSB();
+    }
+
+    // Reopen the device now that we're done
+    device->dev = SDL_hid_open_path(device->path);
+    if (!device->dev) {
+        return false;
+    }
+#endif // HAVE_LIBUSB
+
+    Uint8 initMagic = 0x13;
+    if (SDL_hid_write(device->dev, &initMagic, sizeof(initMagic)) != sizeof(initMagic)) {
+        SDL_LogDebug(SDL_LOG_CATEGORY_INPUT,
+                     "HIDAPI_DriverGameCube_InitDevice(): Couldn't initialize WUP-028");
+        return false;
+    }
+
+    // Wait for the adapter to initialize
+    SDL_Delay(10);
+
+    return true;
+}
+
 static bool HIDAPI_DriverGameCube_InitDevice(SDL_HIDAPI_Device *device)
 {
     SDL_DriverGameCube_Context *ctx;
@@ -106,46 +187,28 @@ static bool HIDAPI_DriverGameCube_InitDevice(SDL_HIDAPI_Device *device)
     Uint8 *curSlot;
     Uint8 i;
     int size;
-    Uint8 initMagic = 0x13;
     Uint8 rumbleMagic = 0x11;
 
-#ifdef HAVE_ENABLE_GAMECUBE_ADAPTORS
-    SDL_EnableGameCubeAdaptors();
-#endif
-
     ctx = (SDL_DriverGameCube_Context *)SDL_calloc(1, sizeof(*ctx));
     if (!ctx) {
         return false;
     }
     device->context = ctx;
 
-    ctx->joysticks[0] = 0;
-    ctx->joysticks[1] = 0;
-    ctx->joysticks[2] = 0;
-    ctx->joysticks[3] = 0;
     ctx->rumble[0] = rumbleMagic;
-    ctx->useRumbleBrake = false;
 
     if (device->vendor_id != USB_VENDOR_NINTENDO) {
         ctx->pc_mode = true;
     }
 
     if (ctx->pc_mode) {
-        for (i = 0; i < MAX_CONTROLLERS; ++i) {
-            ResetAxisRange(ctx, i);
-            HIDAPI_JoystickConnected(device, &ctx->joysticks[i]);
-        }
+        ResetAxisRange(ctx, 0);
+        HIDAPI_JoystickConnected(device, &ctx->joysticks[0]);
     } else {
-        // This is all that's needed to initialize the device. Really!
-        if (SDL_hid_write(device->dev, &initMagic, sizeof(initMagic)) != sizeof(initMagic)) {
-            SDL_LogDebug(SDL_LOG_CATEGORY_INPUT,
-                         "HIDAPI_DriverGameCube_InitDevice(): Couldn't initialize WUP-028");
+        if (!HIDAPI_DriverGameCube_EnableAdapter(device)) {
             return false;
         }
 
-        // Wait for the adapter to initialize
-        SDL_Delay(10);
-
         // Add all the applicable joysticks
         while ((size = SDL_hid_read_timeout(device->dev, packet, sizeof(packet), 0)) > 0) {
 #ifdef DEBUG_GAMECUBE_PROTOCOL
@@ -204,69 +267,61 @@ static void HIDAPI_DriverGameCube_SetDevicePlayerIndex(SDL_HIDAPI_Device *device
 {
 }
 
-static void HIDAPI_DriverGameCube_HandleJoystickPacket(SDL_HIDAPI_Device *device, SDL_DriverGameCube_Context *ctx, const Uint8 *packet, int size)
+static void HIDAPI_DriverGameCube_HandleJoystickPacket(SDL_HIDAPI_Device *device, SDL_DriverGameCube_Context *ctx, const Uint8 *packet, bool invert_c_stick)
 {
     SDL_Joystick *joystick;
-    Uint8 i, v;
+    const Uint8 i = 0;  // We have a separate context for each connected controller in PC mode, just use the first index
+    Uint8 v;
     Sint16 axis_value;
     Uint64 timestamp = SDL_GetTicksNS();
 
-    if (size != 10) {
-        return; // How do we handle this packet?
-    }
-
-    i = packet[0] - 1;
-    if (i >= MAX_CONTROLLERS) {
-        return; // How do we handle this packet?
-    }
-
     joystick = SDL_GetJoystickFromID(ctx->joysticks[i]);
     if (!joystick) {
         // Hasn't been opened yet, skip
         return;
     }
 
-#define READ_BUTTON(off, flag, button)  \
-    SDL_SendJoystickButton(             \
-        timestamp,                      \
-        joystick,                       \
-        button,                         \
+#define READ_BUTTON(off, flag, button) \
+    SDL_SendJoystickButton(            \
+        timestamp,                     \
+        joystick,                      \
+        button,                        \
         ((packet[off] & flag) != 0));
-    READ_BUTTON(1, 0x02, 0) // A
-    READ_BUTTON(1, 0x04, 1) // B
-    READ_BUTTON(1, 0x08, 3) // Y
-    READ_BUTTON(1, 0x01, 2) // X
-    READ_BUTTON(2, 0x80, 4) // DPAD_LEFT
-    READ_BUTTON(2, 0x20, 5) // DPAD_RIGHT
-    READ_BUTTON(2, 0x40, 6) // DPAD_DOWN
-    READ_BUTTON(2, 0x10, 7) // DPAD_UP
-    READ_BUTTON(2, 0x02, 8) // START
-    READ_BUTTON(1, 0x80, 9) // RIGHTSHOULDER
+    READ_BUTTON(0, 0x02, 0) // A
+    READ_BUTTON(0, 0x04, 1) // B
+    READ_BUTTON(0, 0x08, 3) // Y
+    READ_BUTTON(0, 0x01, 2) // X
+    READ_BUTTON(1, 0x80, 4) // DPAD_LEFT
+    READ_BUTTON(1, 0x20, 5) // DPAD_RIGHT
+    READ_BUTTON(1, 0x40, 6) // DPAD_DOWN
+    READ_BUTTON(1, 0x10, 7) // DPAD_UP
+    READ_BUTTON(1, 0x02, 8) // START
+    READ_BUTTON(0, 0x80, 9) // RIGHTSHOULDER
     /* These two buttons are for the bottoms of the analog triggers.
      * More than likely, you're going to want to read the axes instead!
      * -flibit
      */
-    READ_BUTTON(1, 0x20, 10) // TRIGGERRIGHT
-    READ_BUTTON(1, 0x10, 11) // TRIGGERLEFT
+    READ_BUTTON(0, 0x20, 10) // TRIGGERRIGHT
+    READ_BUTTON(0, 0x10, 11) // TRIGGERLEFT
 #undef READ_BUTTON
 
-#define READ_AXIS(off, axis, invert)                                                \
-    v = invert ? (0xff - packet[off]) : packet[off];                                \
-    if (v < ctx->min_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis])                      \
-        ctx->min_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis] = v;                      \
-    if (v > ctx->max_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis])                      \
-        ctx->max_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis] = v;                      \
+#define READ_AXIS(off, axis, invert)                                                                                                                                             \
+    v = (invert) ? (0xff - packet[off]) : packet[off];                                                                                                                           \
+    if (v < ctx->min_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis])                                                                                                                    \
+        ctx->min_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis] = v;                                                                                                                    \
+    if (v > ctx->max_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis])                                                                                                                    \
+        ctx->max_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis] = v;                                                                                                                    \
     axis_value = (Sint16)HIDAPI_RemapVal(v, ctx->min_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis], ctx->max_axis[i * SDL_GAMEPAD_AXIS_COUNT + axis], SDL_MIN_SINT16, SDL_MAX_SINT16); \
-    SDL_SendJoystickAxis(                                                        \
-        timestamp,                                                                  \
-        joystick,                                                                   \
+    SDL_SendJoystickAxis(                                                                                                                                                        \
+        timestamp,                                                                                                                                                               \
+        joystick,                                                                                                                                                                \
         axis, axis_value);
-    READ_AXIS(3, SDL_GAMEPAD_AXIS_LEFTX, 0)
-    READ_AXIS(4, SDL_GAMEPAD_AXIS_LEFTY, 1)
-    READ_AXIS(6, SDL_GAMEPAD_AXIS_RIGHTX, 0)
-    READ_AXIS(5, SDL_GAMEPAD_AXIS_RIGHTY, 1)
-    READ_AXIS(7, SDL_GAMEPAD_AXIS_LEFT_TRIGGER, 0)
-    READ_AXIS(8, SDL_GAMEPAD_AXIS_RIGHT_TRIGGER, 0)
+    READ_AXIS(2, SDL_GAMEPAD_AXIS_LEFTX, 0)
+    READ_AXIS(3, SDL_GAMEPAD_AXIS_LEFTY, 1)
+    READ_AXIS(5, SDL_GAMEPAD_AXIS_RIGHTX, invert_c_stick ? 1 : 0)
+    READ_AXIS(4, SDL_GAMEPAD_AXIS_RIGHTY, invert_c_stick ? 0 : 1)
+    READ_AXIS(6, SDL_GAMEPAD_AXIS_LEFT_TRIGGER, 0)
+    READ_AXIS(7, SDL_GAMEPAD_AXIS_RIGHT_TRIGGER, 0)
 #undef READ_AXIS
 }
 
@@ -365,7 +420,18 @@ static bool HIDAPI_DriverGameCube_UpdateDevice(SDL_HIDAPI_Device *device)
         HIDAPI_DumpPacket("Nintendo GameCube packet: size = %d", packet, size);
 #endif
         if (ctx->pc_mode) {
-            HIDAPI_DriverGameCube_HandleJoystickPacket(device, ctx, packet, size);
+            if (size == 10) {
+                // This is the older firmware
+                // The first byte is the index of the connected controller
+                // The C stick has an inverted value range compared to the left stick
+                HIDAPI_DriverGameCube_HandleJoystickPacket(device, ctx, &packet[1], true);
+            } else if (size == 9) {
+                // This is the newer firmware (version 0x7)
+                // The C stick has the same value range compared to the left stick
+                HIDAPI_DriverGameCube_HandleJoystickPacket(device, ctx, packet, false);
+            } else {
+                // How do we handle this packet?
+            }
         } else {
             HIDAPI_DriverGameCube_HandleNintendoPacket(device, ctx, packet, size);
         }
