@@ -24,8 +24,6 @@
 
 #include "SDL_internal.h"
 
-#include "../SDL_sysrender.h"   // For SDL_TextureAddressMode
-
 extern bool SDL_SW_FillTriangle(SDL_Surface *dst,
                                 SDL_Point *d0, SDL_Point *d1, SDL_Point *d2,
                                 SDL_BlendMode blend, SDL_Color c0, SDL_Color c1, SDL_Color c2);
@@ -35,7 +33,8 @@ extern bool SDL_SW_BlitTriangle(SDL_Surface *src,
                                 SDL_Surface *dst,
                                 SDL_Point *d0, SDL_Point *d1, SDL_Point *d2,
                                 SDL_Color c0, SDL_Color c1, SDL_Color c2,
-                                SDL_TextureAddressMode texture_address_mode);
+                                SDL_TextureAddressMode texture_address_mode_u,
+                                SDL_TextureAddressMode texture_address_mode_v);
 
 extern void trianglepoint_2_fixedpoint(SDL_Point *a);
 
