@@ -22,8 +22,17 @@
 
 typedef struct WIN_GameInputData WIN_GameInputData;
 
+// Set up for C function definitions, even when using C++
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern bool WIN_InitGameInput(SDL_VideoDevice *_this);
 extern bool WIN_UpdateGameInputEnabled(SDL_VideoDevice *_this);
 extern void WIN_UpdateGameInput(SDL_VideoDevice *_this);
 extern void WIN_QuitGameInput(SDL_VideoDevice *_this);
 
+// Ends C function definitions when using C++
+#ifdef __cplusplus
+}
+#endif
