@@ -552,6 +552,7 @@ struct SDL_VideoData
     /* *INDENT-OFF* */ // clang-format off
     BOOL (WINAPI *GetPointerType)(UINT32 pointerId, POINTER_INPUT_TYPE *pointerType);
     BOOL (WINAPI *GetPointerPenInfo)(UINT32 pointerId, POINTER_PEN_INFO *penInfo);
+    BOOL (WINAPI *GetPointerDeviceRects)(HANDLE device, RECT *pointerDeviceRect, RECT *displayRect);
     /* *INDENT-ON* */ // clang-format on
 
     // DPI functions
@@ -590,8 +591,10 @@ struct SDL_VideoData
     SDL_Point last_raw_mouse_position;
     bool raw_mouse_enabled;
     bool raw_keyboard_enabled;
+    bool raw_keyboard_flag_nohotkeys;
     bool pending_E1_key_sequence;
     Uint32 raw_input_enabled;
+    SDL_PenID raw_input_fake_pen_id;
 
     WIN_GameInputData *gameinput_context;
 
